import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { HomePageSectionFourSliderService } from "./home-page-section-four-slider.service";
import { CreateHomePageSectionFourSliderDto } from "./dto/create-home-page-section-four-slider.dto";
import { UpdateHomePageSectionFourSliderDto } from "./dto/update-home-page-section-four-slider.dto";

@Controller({ path: "home-page-section-four-slider", version: Constants.API_VERSION_1 })

export class HomePageSectionFourSliderController {
    constructor(private readonly service: HomePageSectionFourSliderService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateHomePageSectionFourSliderDto) {
        return this.service.create(dto);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    update(
      @Param("id") id: string,
      @Body() dto: UpdateHomePageSectionFourSliderDto) {
        return this.service.update(id, dto);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
