import { Controller, Get, Post, Body, HttpCode, UseGuards, HttpStatus, UseInterceptors, UploadedFiles } from '@nestjs/common';
import { SectionOneService } from './section-one.service';
import { SectionOneDto } from './dto/section-one.dto';
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";

@Controller({ path: "section-one", version: Constants.API_VERSION_1 })
export class SectionOneController {
    constructor(private readonly service: SectionOneService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "backgroundImage", maxCount: 1 }
        ]
    ))
    updateOrCreateData(
        @Body() dto: SectionOneDto,
        @UploadedFiles() files: {
            backgroundImage: UploadedMulterFileI
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }
}
