import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { SectionOneDto } from './dto/section-one.dto';
import { SectionOneRepository } from "./section-one.repository";
import { InjectRepository } from "@nestjs/typeorm";
import { SectionOne } from "./entities/section-one.entity";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class SectionOneService {
    constructor(
        @InjectRepository(SectionOneRepository)
        private readonly repository: SectionOneRepository,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async updateOrCreateData(
        dto: SectionOneDto,
        files: {
            backgroundImage: UploadedMulterFileI
        }
    ): Promise<SectionOne | Error> {
        if (files && files.backgroundImage) {
            const backgroundImage: any = await this.doSpaceService.uploadFile(files.backgroundImage[0], "SectionOneCMS/");
            dto.backgroundImage = backgroundImage;
        }

        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<SectionOne[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
