import { Controller, Get, Post, Body, HttpCode, HttpStatus, UseGuards, UseInterceptors, UploadedFiles } from '@nestjs/common';
import { SectionThreeService } from './section-three.service';
import { SectionThreeDto } from './dto/section-three.dto';
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { Constants } from "src/utils/constants";

@Controller({path: "section-three", version: Constants.API_VERSION_1 })
export class SectionThreeController {
    constructor(private readonly service: SectionThreeService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "imageOne", maxCount: 1 },
            { name: "imageTwo", maxCount: 1 },
            { name: "imageThree", maxCount: 1 }
        ]
    ))
    updateOrCreateData(
        @Body() dto: SectionThreeDto,
        @UploadedFiles() files: {
            imageOne: UploadedMulterFileI,
            imageTwo: UploadedMulterFileI,
            imageThree: UploadedMulterFileI
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }
}
