import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { SectionTwoBottomDto } from './dto/section-two-bottom.dto';
import { SectionTwoBottomRepository } from "./section-two-bottom.repository";
import { InjectRepository } from "@nestjs/typeorm";
import { SectionTwoBottom } from "./entities/section-two-bottom.entity";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";

@Injectable()
export class SectionTwoBottomService {
    constructor(
        @InjectRepository(SectionTwoBottomRepository)
        private readonly repository: SectionTwoBottomRepository
    ) { }

    async updateOrCreateData(dto: SectionTwoBottomDto): Promise<SectionTwoBottom | Error> {

        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<SectionTwoBottom[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
