import { Controller, Get, Post, Body, HttpCode, HttpStatus, UseGuards, UseInterceptors, UploadedFiles } from '@nestjs/common';
import { SectionTwoTopService } from './section-two-top.service';
import { SectionTwoTopDto } from './dto/section-two-top.dto';
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { Constants } from "src/utils/constants";

@Controller({path: "section-two-top", version: Constants.API_VERSION_1 })
export class SectionTwoTopController {
    constructor(private readonly service: SectionTwoTopService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "video", maxCount: 1 },
        ]
    ))
    updateOrCreateData(
        @Body() dto: SectionTwoTopDto,
        @UploadedFiles() files: {
            video: UploadedMulterFileI
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }
}
