import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";
import { DoSpacesService } from "../SpacesModule/SpacesService/doSpacesService";
import { AboutPageSectionTwoRepository } from "./about-page-section-two.repository";
import { CreateAboutPageSectionTwoDto } from "./dto/create-about-page-section-two.dto";
import { UpdateAboutPageSectionTwoDto } from "./dto/update-about-page-section-two.dto";
import { AboutPageSectionTwo } from "./entities/about-page-section-two.entity";

@Injectable()
export class AboutPageSectionTwoService {
    constructor(
      @InjectRepository(AboutPageSectionTwoRepository)
      private readonly repository: AboutPageSectionTwoRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(dto: CreateAboutPageSectionTwoDto, files: {
        image: UploadedMulterFileI,
    }): Promise<AboutPageSectionTwo | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "AboutPageSectionTwo/");
            dto.image = image;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async update(
      id: string,
      dto: UpdateAboutPageSectionTwoDto,
      files: {
          image: UploadedMulterFileI,
      }
    ): Promise<AboutPageSectionTwo | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "AboutPageSectionTwo/");
            dto.image = image;
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
              Constants.UPDATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async findAll(): Promise<AboutPageSectionTwo[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<AboutPageSectionTwo[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
