import { Injectable } from "@nestjs/common";
import { fetchAllRetailers, fetchOneRetailer, menuType, nearByRetailers } from "src/helper/dutchie-plus-query";
import { getGraphQLClient } from "src/helper/graphql-client-helper";
import { InjectRepository } from "@nestjs/typeorm";
import { RetailerServiceRepository } from "../../retailer-service/retailer-service.repository";


@Injectable()
export class RetailerService {
    constructor(
        @InjectRepository(RetailerServiceRepository)
        private readonly retailerServiceRepo: RetailerServiceRepository
    ) {
    }

    async filterRetailers(data: any, search: string) {
        try {
            let searchList = [];
            for (let i = 0; i < data.length; i++) {
                const name = data[i].name.toLowerCase();
                const address = data[i].address.toLowerCase();
                const res = name.includes(search.toLowerCase()) || address.includes(search.toLowerCase());
                if (res) {
                    searchList.push(data[i]);
                }
            }
            return searchList;
        } catch (e) {
            return e;
        }
    }

    async checkRetailerDifference(data: any): Promise<any | Error> {
        try {
            const retailerService = await this.retailerServiceRepo.findByFilterQuery({});
            if (Object.keys(retailerService).length > 0) {
                const difference = data.retailers.filter((obj: any) => {
                    return (retailerService.findIndex((x) => x.retailerId === obj.id) === -1);
                });
                console.log("Retailer Difference = ", difference);
                for (let i = 0; i < difference.length; i++) {
                    const dtoObj = {
                        retailerId: difference[i].id,
                        retailerName: difference[i].name
                    };
                    await this.retailerServiceRepo.createEntity(dtoObj);
                }
            } else {
                for (let i = 0; i < data.retailers.length; i++) {
                    const dtoObj = {
                        retailerId: data.retailers[i].id,
                        retailerName: data.retailers[i].name
                    };
                    await this.retailerServiceRepo.createEntity(dtoObj);
                }
            }
        } catch (e) {
            console.log(e);
            return e;
        }
    }

    async findAll(type: string, search: string) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllRetailers();
            const data = await graphQLClient.request(query);
            // store retailers in local storage
            await this.checkRetailerDifference(data);
            // check retailer type
            if (type === "DELIVERY") {
                let deliveryList = [];
                const retailers = data.retailers;
                // console.log(retailers[0].hours.delivery)
                for (let i = 0; i < retailers.length; i++) {
                    if (retailers[i].deliverySettings.afterHoursOrderingForDelivery) {
                        const delivery = retailers[i].hours.delivery;
                        if (
                            delivery.Sunday.active &&
                            delivery.Monday.active &&
                            delivery.Tuesday.active &&
                            delivery.Wednesday.active &&
                            delivery.Thursday.active &&
                            delivery.Friday.active &&
                            delivery.Saturday.active
                        ) {
                            deliveryList.push(retailers[i]);
                        }
                    }
                }

                if (typeof search !== "undefined") {
                    return await this.filterRetailers(deliveryList, search);
                } else {
                    return deliveryList;
                }
            } else if (type === "PICKUP") {
                let pickupList = [];
                const retailers = data.retailers;
                for (let i = 0; i < retailers.length; i++) {
                    if (retailers[i].deliverySettings.afterHoursOrderingForPickup) {
                        const pickup = retailers[i].hours.pickup;
                        if (
                            pickup.Sunday.active &&
                            pickup.Monday.active &&
                            pickup.Tuesday.active &&
                            pickup.Wednesday.active &&
                            pickup.Thursday.active &&
                            pickup.Friday.active &&
                            pickup.Saturday.active
                        ) {
                            pickupList.push(retailers[i]);
                        }
                    }
                }

                if (typeof search !== "undefined") {
                    return await this.filterRetailers(pickupList, search);
                } else {
                    return pickupList;
                }
            } else {
                let allList = [];
                const retailers = data.retailers;
                for (let i = 0; i < retailers.length; i++) {
                    if (retailers[i].deliverySettings.afterHoursOrderingForDelivery || retailers[i].deliverySettings.afterHoursOrderingForPickup) {
                        const retailerService = await this.retailerServiceRepo.findOne({
                            where: {
                                retailerId: retailers[i].id,
                                isDeleted: false
                            }
                        });

                        if (retailerService) {
                            retailers[i]["contactNumber"] = retailerService.retailerMobileNumber ? retailerService.retailerMobileNumber : "";
                        }
                        else {
                            retailers[i]["contactNumber"] = "";
                        }

                        allList.push(retailers[i]);
                    }
                }
                if (typeof search !== "undefined") {
                    return await this.filterRetailers(allList, search);
                } else {
                    return allList;
                }
            }
        } catch (e) {
            return e;
        }
    }

    async findOne(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchOneRetailer();
            const data = await graphQLClient.request(query, variables);
            if (data) {
                const retailerService = await this.retailerServiceRepo.findOne({
                    where: {
                        retailerId: data.retailer.id,
                        isDeleted: false
                    }
                });
                if (retailerService) {
                    await this.retailerServiceRepo.update(retailerService.id, {
                        accessCount: retailerService.accessCount + 1
                    });
                }
                return data;
            }
        } catch (e) {
            return e;
        }
    }

    async menuType(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = menuType();
            const data = await graphQLClient.request(query, variables);
            return data["retailer"];
        } catch (e) {
            return e;
        }
    }

    async countAllRetailer() {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllRetailers();
            const data = await graphQLClient.request(query);
            const count = Object.keys(data.retailers).length;
            return count;
        } catch (e) {
            return e;
        }
    }


    async filterRetailer(variables: any, type: string, search: string) {
        try {
            let nearByRetailerData = [];
            const graphQLClient = getGraphQLClient();
            if (type === "Closest Location") {
                const query = nearByRetailers();
                const data = await graphQLClient.request(query, variables);
                for (let index = 0; index < data.retailersNearLocation.length; index++) {
                    const retailerService = await this.retailerServiceRepo.findOne({
                        where: {
                            retailerId: data.retailersNearLocation[index].retailer.id,
                            isDeleted: false
                        }
                    });

                    if (retailerService) {
                        data.retailersNearLocation[index].retailer["contactNumber"] = retailerService.retailerMobileNumber ? retailerService.retailerMobileNumber : "";
                    }
                    else {
                        data.retailersNearLocation[index].retailer["contactNumber"] = "";
                    }
                    nearByRetailerData.push(data.retailersNearLocation[index].retailer);
                }
                if (typeof search !== "undefined") {
                    return await this.filterRetailers(nearByRetailerData, search);
                }
                else {
                    return nearByRetailerData;
                }
            }
            else {
                const query = fetchAllRetailers();
                const data = await graphQLClient.request(query);
                // store retailers in local storage
                await this.checkRetailerDifference(data);

                let allList = [];
                const retailers = data.retailers;
                for (let i = 0; i < retailers.length; i++) {
                    if (retailers[i].deliverySettings.afterHoursOrderingForDelivery || retailers[i].deliverySettings.afterHoursOrderingForPickup) {
                        const retailerService = await this.retailerServiceRepo.findOne({
                            where: {
                                retailerId: retailers[i].id,
                                isDeleted: false
                            }
                        });

                        if (retailerService) {
                            retailers[i]["contactNumber"] = retailerService.retailerMobileNumber ? retailerService.retailerMobileNumber : "";
                        }
                        else {
                            retailers[i]["contactNumber"] = "";
                        }

                        allList.push(retailers[i]);
                    }
                }

                if (typeof search !== "undefined" && type === "ZipOrLocation") {
                    return await this.filterRetailers(allList, search);
                }
                else {
                    return allList;
                }
            }
        } catch (e) {
            return e;
        }
    }
}
