import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { CreateProductServiceDto } from "./dto/create-product-service.dto";
import { UpdateProductServiceDto } from "./dto/update-product-service.dto";
import { ProductServiceRepository } from "./product-service.repository";
import { ResponseUtils } from "src/utils/response.utils";
import { ProductService } from "./entities/product-service.entity";
import { getGraphQLClient } from "src/helper/graphql-client-helper";
import {
    fetchAllProductCategory,
    fetchAllProductsForScheduler,
    fetchAllRetailers,
    fetchOneProduct,
    fetchProductsCount
} from "src/helper/dutchie-plus-query";
import { ElasticsearchService } from "@nestjs/elasticsearch";
import { SearchProductServiceDto } from "./dto/search-product-service.dto";
import { CreateProductCategoryDto } from "../product-category/dto/create-product-category.dto";
import { InjectRepository } from "@nestjs/typeorm";
import { ProductCategoryRepository } from "../product-category/product-category.repository";

@Injectable()
export class ProductServiceService {
    constructor(
      private readonly repository: ProductServiceRepository,
      private readonly esService: ElasticsearchService,
      @InjectRepository(ProductCategoryRepository)
      private readonly productCatRepo: ProductCategoryRepository
    ) {
    }

    async createUpdateDutchiePlusProducts() {
        try {
            console.log(">>>Run Create or Update Dutchie Plus Products manually<<<");
            const graphQLClient = getGraphQLClient();
            const query = fetchAllRetailers();
            const data = await graphQLClient.request(query);
            const retailers = data.retailers;
            if (retailers.length > 0) {
                for (let i = 0; i < retailers.length; i++) {
                    // find all products count based on retailers
                    const variables = {
                        retailerId: retailers[i].id
                    };
                    const fetchProductsCountQuery = fetchProductsCount();
                    const productCountData = await graphQLClient.request(
                      fetchProductsCountQuery,
                      variables
                    );
                    const productCount = productCountData["menu"].productsCount;
                    if (productCount > 0) {
                        // fetch all products from dutchie plus
                        const productsVariable = {
                            retailerId: retailers[i].id,
                            pagination: {
                                offset: 0,
                                limit: productCount
                            }
                        };
                        const fetchProductsQuery = fetchAllProductsForScheduler();
                        const productData = await graphQLClient.request(
                          fetchProductsQuery,
                          productsVariable
                        );
                        // fetch local database data
                        const productService = await this.repository.findAll();
                        if (Object.keys(productService).length > 0) {
                            // get difference between local products and dutchie products
                            const differenceArray = productData["menu"]["products"].filter(
                              (obj: any) => {
                                  return (
                                    productService.findIndex((o) => o.productId === obj.id) === -1
                                  );
                              }
                            );
                            console.log("Product Difference=", differenceArray);
                            for (let j = 0; j < differenceArray.length; j++) {
                                const brand = differenceArray[j].brand !== null ? differenceArray[j].brand.name : "";
                                const productServiceDto = {
                                    retailerId: retailers[i].id,
                                    retailerName: retailers[i].name,
                                    productId: differenceArray[j].id,
                                    productName: differenceArray[j].name,
                                    productSlug: differenceArray[j].slug,
                                    image: differenceArray[j].image,
                                    brandName: brand,
                                    category: differenceArray[j].category,
                                    variants: differenceArray[j].variants.length
                                };
                                await this.repository.createEntity(
                                  productServiceDto as CreateProductServiceDto
                                );
                            }
                        } else {
                            // if no product in database create an entity
                            for (let k = 0; k < productData['menu']['products'].length; k++) {
                                const brand = productData["menu"]["products"][k].brand !== null ? productData["menu"]["products"][k].brand.name : "";
                                const productServiceDto = {
                                    retailerId: retailers[i].id,
                                    retailerName: retailers[i].name,
                                    productId: productData["menu"]["products"][k].id,
                                    productName: productData["menu"]["products"][k].name,
                                    productSlug: productData["menu"]["products"][k].slug,
                                    image: productData["menu"]["products"][k].image,
                                    brandName: brand,
                                    category: productData["menu"]["products"][k].category,
                                    variants: productData["menu"]["products"][k].variants.length
                                };
                                await this.repository.createEntity(
                                  productServiceDto as CreateProductServiceDto
                                );
                            }
                        }
                    }
                }
            } else {
                console.log("Retailers Not Found!");
                return ResponseUtils.errorResponseHandler(400, "Retailers Not Found!");
            }
        } catch (e) {
            console.log(e);
            return e;
        }
    }


    async createUpdateDutchiePlusCategory() {
        try {
            console.log(">>>Run Create or Update Dutchie Plus Category Manually<<<");
            const graphQLClient = getGraphQLClient();
            const query = fetchAllRetailers();
            const data = await graphQLClient.request(query);
            const retailers = data.retailers;
            if (retailers.length > 0) {
                let category = [];
                for (let i = 0; i < retailers.length; i++) {
                    // find all category based on reatilers
                    const variables = {
                        retailerId: retailers[i].id
                    };
                    const query = fetchAllProductCategory();
                    const data = await graphQLClient.request(query, variables);
                    const categories = data["menu"]["products"];

                    // remove duplicate product category from api response
                    for (let i = 0; i < categories.length; i++) {
                        if (!category.includes(categories[i].category)) {
                            category.push(categories[i].category);
                        }
                    }
                }
                // fetch all product category
                const productCat = await this.productCatRepo.findAll();
                // console.log({productCat});
                if (Object.keys(productCat).length > 0) {
                    const difference = category.filter(
                      (obj: any) => {
                          return (productCat.findIndex((c) => c.categoryName === obj) === -1);
                      }
                    );
                    console.log("Product Categories Difference = ", difference);
                    // if difference found then create entities with difference product category
                    for (let j = 0; j < difference.length; j++) {
                        const dto = {
                            categoryName: difference[j]
                        };
                        const createProCat = await this.productCatRepo.createEntity(
                          dto as CreateProductCategoryDto
                        );
                        console.log({ createProCat });
                    }
                } else {
                    // if product categories table is empty in our local database then create all the category in local
                    for (let j = 0; j < category.length; j++) {
                        const dto = {
                            categoryName: category[j]
                        };
                        const createProCat = await this.productCatRepo.createEntity(
                          dto as CreateProductCategoryDto
                        );
                        console.log({ createProCat });
                    }
                }
            } else {
                console.log("Retailers Not Found!");
                return ResponseUtils.errorResponseHandler(400, "Retailers Not Found!");
            }
        } catch (e) {
            console.log(e);
            return e;
        }
    }

    async productSynchronize(): Promise<any | Error> {
        try {
            await this.createUpdateDutchiePlusProducts();
            await this.createUpdateDutchiePlusCategory();
            return ResponseUtils.successResponseHandler(200, "Product Synchronize completed Successfully!");
        } catch (e) {
            console.log(e);
            return e;
        }
    }

    async create(dto: CreateProductServiceDto): Promise<any | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(Constants.CREATE_FAILED, HttpStatus.BAD_REQUEST);
        }
        return ResponseUtils.successResponseHandler(
          201,
          "Products created successfully",
          "data",
          data
        );
    }

    async findAll(): Promise<ProductService[] | Error> {
        try {
            const data = await this.repository.findAll();
            if (!data) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }
            return ResponseUtils.successResponseHandler(
              200,
              "Products list fetched successfully",
              "data",
              data
            );
        } catch (error) {
            console.log(error);
            return error;
        }
    }

    async findOne(slug: string): Promise<any | Error> {
        try {
            const data = await this.repository.findOneEntity(slug);
            if (data) {
                const variables = {
                    retailerId: data.retailerId,
                    productId: data.productId
                };
                const graphQLClient = getGraphQLClient();
                const query = fetchOneProduct();
                return await graphQLClient.request(query, variables);
            } else {
                console.log(">>Fetch Products From Dutchie<<");
                await this.createUpdateDutchiePlusProducts();
                await this.findOne(slug);
            }
        } catch (error) {
            console.log(error);
            return error;
        }
    }

    async update(
      id: string,
      dto: UpdateProductServiceDto
    ): Promise<ProductService | Error> {
        const data = await this.repository.updateProductCategory(id, dto);
        if (!data) {
            throw new HttpException(Constants.UPDATE_FAILED, HttpStatus.BAD_REQUEST);
        }
        return ResponseUtils.successResponseHandler(
          200,
          "Products updated successfully",
          "data",
          data
        );
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.BAD_REQUEST);
        }
        return ResponseUtils.buildDeletedData(
          data,
          200,
          "Products deleted successfully"
        );
    }

    //#region Elastic Search

    public async indexIsExists(index: string) {
        const isExists = await this.esService.indices.exists({ index });
        return isExists;
    }

    public async deleteIndex(index: string) {
        const isExists = await this.indexIsExists(index);
        if (isExists) {
            await this.esService.indices.delete({ index });
        }
    }

    public async createIndex(index: string, body: any) {
        const isExists = await this.indexIsExists(index);
        if (!isExists) {
            const data = await this.esService.indices.create({ index, body });
            return data.acknowledged;
        }
        return false;
    }

    public async indexData(index: string, body: any) {
        return await this.esService.index({ index, body });
    }

    public async indexBulkData(indexValue: string) {
        const res: any = await this.findAll();
        const products: ProductService[] = res?.data;

        await this.deleteIndex(indexValue);

        const body = {
            settings: {
                analysis: {
                    filter: {
                        autocomplete_filter: {
                            type: "edge_ngram",
                            min_gram: 1,
                            max_gram: 15
                        }
                    },
                    analyzer: {
                        autocomplete: {
                            type: "custom",
                            tokenizer: "standard",
                            filter: ["lowercase", "autocomplete_filter"]
                        }
                    }
                }
            },

            mappings: {
                properties: {
                    id: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    category: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    image: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    productId: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    productName: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    brandName: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    brandNameAndProductNameAndCategoryName: {
                        type: "text",
                        analyzer: "autocomplete",
                        search_analyzer: "standard",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 512
                            }
                        }
                    },
                    productSlug: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    retailerId: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    retailerName: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    variants: {
                        type: "long"
                    },
                    isDeleted: {
                        type: "boolean"
                    },
                    createdAt: {
                        type: "date"
                    },
                    updatedAt: {
                        type: "date"
                    }

                }
            }
        };
        await this.createIndex(indexValue, body);

        const operations = products.flatMap((doc) => [
            { index: { _index: indexValue, _id: doc.id } },
            {
                ...doc,
                brandNameAndProductNameAndCategoryName: doc.productName.trim().startsWith(doc.brandName.trim())
                  ? `${doc.productName} ${doc.category}`
                  : `${doc.brandName} ${doc.productName} ${doc.category}`
            }
        ]);
        return await this.esService.bulk({ refresh: true, operations });
    }

    public async search(index: string, searchProductServiceDto: SearchProductServiceDto) {
        const { search, customField, page, limit, retailerId } = searchProductServiceDto;

        let exactsMatch = [];
        if (retailerId && retailerId != "") {
            const retailersMatch = {
                match_phrase: {
                    retailerId: retailerId.toLowerCase()
                }
            };
            exactsMatch = [...exactsMatch, retailersMatch];
        }

        let fields: Array<string> = ["brandNameAndProductNameAndCategoryName"];
        if (customField && customField?.length) {
            fields = customField;
        }

        const result = await this.esService.search<any>({
            index,
            body: {
                query: {
                    bool: {
                        must: [
                            ...exactsMatch,
                            {
                                match: {
                                    brandNameAndProductNameAndCategoryName: {
                                        query: search,
                                        fuzziness: "AUTO",
                                        fuzzy_transpositions: false,
                                        prefix_length: 3
                                    }
                                }
                            }
                        ]
                    }
                },
                highlight: {
                    fields: {
                        brandNameAndProductNameAndCategoryName: {
                            pre_tags: ["<b>"],
                            post_tags: ["</b>"]
                        }
                    }
                }
            },
            // from: page && page > 0 ? page - 1 : 0,
            size: limit && limit > 0 ? limit : 1000
        });
        const hits = result.hits.hits;
        const pageCount: any = result.hits.total;

        const products = hits.map((item: any) => {
            return {
                ...item._source,
                highlight: item?.highlight?.brandNameAndProductNameAndCategoryName?.[0]
            };
        });
        const data = { products, total: (pageCount?.value ?? pageCount) };

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
          200,
          "Products list fetched successfully",
          "data",
          data
        );
    }

    public async searchDetails(index: string, searchProductServiceDto: SearchProductServiceDto) {
        const { search, customField, page, limit, retailerId } = searchProductServiceDto;

        let exactsMatch = [];
        if (retailerId && retailerId != "") {
            const retailersMatch = {
                match_phrase: {
                    retailerId: retailerId.toLowerCase()
                }
            };
            exactsMatch = [...exactsMatch, retailersMatch];
        }

        let fields: Array<string> = ["brandNameAndProductNameAndCategoryName"];
        if (customField && customField?.length) {
            fields = customField;
        }

        const result = await this.esService.search<any>({
            index,
            body: {
                query: {
                    bool: {
                        must: [
                            ...exactsMatch,
                            {
                                match: {
                                    brandNameAndProductNameAndCategoryName: {
                                        query: search,
                                        fuzziness: "AUTO",
                                        fuzzy_transpositions: false,
                                        prefix_length: 3
                                    }
                                }
                            }
                        ]
                    }
                }
            },
            // from: page ? page : 0,
            size: 1000
        });

        const data = result.hits.hits;
        const start = page ? page : 0;
        const end = limit > 0 ? limit : 20;
        const hits = data.slice(start * end, (start + 1) * end);
        const pageCount: any = result.hits.total;
        let menu = {
            weights: [],
            brands: [],
            productsCount: pageCount?.value ?? pageCount,
            products: [],
            subCategory: []
        };
        if (hits.length > 0) {
            const graphQLClient = getGraphQLClient();
            let BrandHashSet = new Set();
            let subCategoryHashSet = new Set();
            let weightsHashSet = new Set();
            const allApi = [];

            for (let i = 0; i < hits.length; i++) {
                const productId = hits[i]._source.productId;
                const variables = { retailerId, productId };
                const query = fetchOneProduct();
                const value = graphQLClient.request(query, variables);
                allApi.push(value.catch((error: any) => {
                    console.error(error);
                    return Promise.resolve();
                }));
            }
            // const results = await Promise.all(allApi);
            await Promise.all(allApi)
              .then((results) => {
                  for (let j = 0; j < results.length; j++) {
                      results[j].product.variants.forEach((x: { option: any; }) => {
                          if (!weightsHashSet.has(x.option)) {
                              weightsHashSet.add(x.option);
                              menu.weights.push(x.option);
                          }
                      });
                      if (!BrandHashSet.has(results[j].product.brand.id)) {
                          BrandHashSet.add(results[j].product.brand.id);
                          menu.brands.push(results[j].product.brand);
                      }
                      menu.products.push(results[j].product);
                      if (!subCategoryHashSet.has(results[j].product.subcategory)) {
                          subCategoryHashSet.add(results[j].product.subcategory);
                          menu.subCategory.push(results[j].product.subcategory);
                      }
                  }
              })
              .catch(error => {
                  console.log(error);
              })
              .finally(() => {
                  return { menu };
              });
        }

        return { menu };
    }

    //#endregion Elastic Search
}
