import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { SocialPackService } from "./social-pack.service";
import { CreateSocialPackDto } from "./dto/create-social-pack.dto";
import { UpdateSocialPackDto } from "./dto/update-social-pack.dto";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";

@Controller({ path: "social-pack", version: Constants.API_VERSION_1 })

export class SocialPackController {
    constructor(private readonly service: SocialPackService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateSocialPackDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    update(@Param("id") id: string, @Body() dto: UpdateSocialPackDto) {
        return this.service.update(id, dto);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
